==> widget_code_input-3.6.1.tar.gz: widget_code_input-3.6.1/src/widget.ts <==
// Copyright (c) Dou Du
// Distributed under the terms of the Modified BSD License.

import {
  DOMWidgetModel,
  DOMWidgetView,
  ISerializers,
} from '@jupyter-widgets/base';

import { MODULE_NAME, MODULE_VERSION } from './version';

// Import the CSS
import '../css/widget.css';

import * as _ from 'underscore';

// Import the CodeMirror library
import CodeMirror from 'codemirror';
import 'codemirror/mode/css/css';
import 'codemirror/mode/javascript/javascript';

// Import the css for the CodeMirror themes
import '../css/midnight.css';
import '../css/monokai.css';
import '../css/eclipse.css';
import '../css/material.css';
import '../css/solarized.css';
import '../css/idea.css';
import '../css/nord.css';

import 'codemirror/mode/python/python.js';

export class WidgetCodeModel extends DOMWidgetModel {
  defaults() {
    return {
      ...super.defaults(),
      _model_name: WidgetCodeModel.model_name,
      _model_module: WidgetCodeModel.model_module,
      _model_module_version: WidgetCodeModel.model_module_version,
      _view_name: WidgetCodeModel.view_name,
      _view_module: WidgetCodeModel.view_module,
      _view_module_version: WidgetCodeModel.view_module_version,
      value: 'Hello World',
    };
  }

  initialize(...args: any[]) {
    const [attributes, options] = args;
    DOMWidgetModel.prototype.initialize.apply(this, [attributes, options]);
    this.attributes['function_body_id'] = _.uniqueId('function_body');
  }

  static serializers: ISerializers = {
    ...DOMWidgetModel.serializers,
    // Add any extra serializers here
  };

  static model_name = 'WidgetCodeModel';
  static model_module = MODULE_NAME;
  static model_module_version = MODULE_VERSION;
  static view_name = 'WidgetCodeView'; // Set to null if no view
  static view_module = MODULE_NAME; // Set to null if no view
  static view_module_version = MODULE_VERSION;
}

export class WidgetCodeView extends DOMWidgetView {
  private mySignatureCodeMirror: any;
  private myDocstringCodeMirror: any;
  private myBodyCodeMirror: any;
  private theme: string = this.model.get('code_theme');

  render() {
    const cssStyles =
      '<style>\
            .forced-indent {width: 4em;}\
            .CodeMirror {border: 1px solid #aaa; height: auto;}\
            .CodeMirror.widget-code-input-signature {border-bottom: 0px;}\
            .CodeMirror.widget-code-input-docstring {border-top: 0px;}\
            .CodeMirror.widget-code-input-body {border-top: 0px;}\
        </style>';

    const theTextareaId = this.model.get('function_body_id');

    this.el.innerHTML =
      cssStyles +
      '<textarea id="' +
      theTextareaId +
      '-signature"></textarea>' +
      '<textarea id="' +
      theTextareaId +
      '-docstring"></textarea>' +
      '<textarea id="' +
      theTextareaId +
      '-body"></textarea>';

    _.defer(() => {
      this.mySignatureCodeMirror = CodeMirror.fromTextArea(
        <HTMLTextAreaElement>(
          document.getElementById(theTextareaId + '-signature')
        ),
        {
          lineNumbers: true,
          firstLineNumber: 1,
          mode: {
            name: 'python',
            version: 3,
            singleLineStringErrors: true,
            matchBrackets: true,
          },
          readOnly: true,
          indentUnit: 4,
          indentWithTabs: false,
        }
      );

      this.mySignatureCodeMirror.setOption('theme', this.theme);
      this.mySignatureCodeMirror.display.wrapper.classList.add(
        'widget-code-input-signature'
      );

      this.myDocstringCodeMirror = CodeMirror.fromTextArea(
        <HTMLTextAreaElement>(
          document.getElementById(theTextareaId + '-docstring')
        ),
        {
          lineNumbers: true,
          //firstLineNumber: // This will be set later by changing the content, this
          // indirectly calls the function to update the line numbers
          mode: {
            name: 'python',
            version: 3,
            singleLineStringErrors: true,
            matchBrackets: true,
          },
          readOnly: true,
          indentUnit: 4,
          indentWithTabs: false,
          gutters: ['CodeMirror-linenumbers', 'forced-indent'],
        }
      );
      // Add CSS
      this.myDocstringCodeMirror.setOption('theme', this.theme);
      this.myDocstringCodeMirror.display.wrapper.classList.add(
        'widget-code-input-docstring'
      );

      this.myBodyCodeMirror = CodeMirror.fromTextArea(
        <HTMLTextAreaElement>document.getElementById(theTextareaId + '-body'),
        {
          lineNumbers: true,
          //firstLineNumber: // This will be set later by changing the content, this
          // indirectly calls the function to update the line numbers
          mode: {
            name: 'python',
            version: 3,
            singleLineStringErrors: true,
            matchBrackets: true,
          },
          readOnly: false,
          indentUnit: 4,
          indentWithTabs: false,
          gutters: ['CodeMirror-linenumbers', 'forced-indent'],
        }
      );

      // Add CSS
      this.myBodyCodeMirror.setOption('theme', this.theme);
      this.myBodyCodeMirror.display.wrapper.classList.add(
        'widget-code-input-body'
      );
      // I need to attach the backboneView, since I'm going to use
      // a CodeMirror event rather than a backboneView, but
      // bodyChange needs to access the Backbone.js View
      this.myBodyCodeMirror._backboneView = this;

      // Set the initial values of the cells
      this.signatureValueChanged();
      this.docstringValueChanged();
      this.bodyValueChanged();

      // When the value is changed in python, update the value in the widget
      // I set this in the 'defer' so that bodyValueChanged is called only
      // when the CodeWidget has been  rendered.
      this.model.on('change:function_name', this.signatureValueChanged, this);
      this.model.on(
        'change:function_parameters',
        this.signatureValueChanged,
        this
      );
      this.model.on('change:docstring', this.docstringValueChanged, this);
      this.model.on('change:function_body', this.bodyValueChanged, this);
      this.model.on('change:code_theme', this.themeChanged, this);

      // For a proper CodeMirror functioning, we use CodeMirror events
      // rather than Backbone.js events.
      // Only the function_body is not read-only, so we need only this
      // event
      this.myBodyCodeMirror.on('change', this.bodyChange);
    });
  }

  themeChanged() {
    this.theme = this.model.get('code_theme');
    this.mySignatureCodeMirror.setOption('theme', this.theme);
    this.myDocstringCodeMirror.setOption('theme', this.theme);
    this.myBodyCodeMirror.setOption('theme', this.theme);
  }

  bodyChange(instance: any, changeObj: any) {
    // On change of the text in the code widget, send back the content
    // to python.
    // Note that this event is triggered by backbone, so "this" is
    // NOT this class but the window; I need to access the view by
    // instance._backboneView that I attached when rendering.
    const currentFunctionBody = instance.getValue();
    instance._backboneView.model.set('function_body', currentFunctionBody);
    instance._backboneView.touch();
    console.log('Done');
  }

  signatureValueChanged() {
    // Set the value from python into the CodeMirror widget in the
    // frontend.
    const newSignature =
      'def ' +
      this.model.get('function_name') +
      '(' +
      this.model.get('function_parameters') +
      '):';
    if (newSignature !== this.mySignatureCodeMirror.getValue()) {
      this.mySignatureCodeMirror.setValue(newSignature);
    }
  }

  docstringValueChanged() {
    // Set the value from python into the CodeMirror widget in the
    // frontend.
    const newDocstring = '"""' + this.model.get('docstring') + '"""';
    if (newDocstring !== this.myDocstringCodeMirror.getValue()) {
      this.myDocstringCodeMirror.setValue(newDocstring);
    }
    this.updateLineNumbers();
  }

  bodyValueChanged() {
    // Set the value from python into the CodeMirror widget in the
    // frontend. Update it only if the content has changed:
    // typing re-fires this event, that would then change the content
    // of the widget, moving the cursor back to the top of the cell
    if (this.model.get('function_body') !== this.myBodyCodeMirror.getValue()) {
      this.myBodyCodeMirror.setValue(this.model.get('function_body'));
    }
  }

  updateLineNumbers() {
    const linesSignature = this.mySignatureCodeMirror
      .getValue()
      .split('\n').length;
    const linesDocstring = this.myDocstringCodeMirror
      .getValue()
      .split('\n').length;
    this.myDocstringCodeMirror.setOption('firstLineNumber', linesSignature + 1);
    this.myBodyCodeMirror.setOption(
      'firstLineNumber',
      linesDocstring + linesSignature + 1
    );
  }
}

==> widget_code_input-3.6.2.tar.gz: widget_code_input-3.6.2/src/widget.ts <==
// Copyright (c) Dou Du
// Distributed under the terms of the Modified BSD License.

import {
  DOMWidgetModel,
  DOMWidgetView,
  ISerializers,
} from '@jupyter-widgets/base';

import { MODULE_NAME, MODULE_VERSION } from './version';

// Import the CSS
import '../css/widget.css';

import * as _ from 'underscore';

// Import the CodeMirror library
import CodeMirror from 'codemirror';
import 'codemirror/mode/css/css';
import 'codemirror/mode/javascript/javascript';

// Import the css for the CodeMirror themes
import '../css/midnight.css';
import '../css/monokai.css';
import '../css/eclipse.css';
import '../css/material.css';
import '../css/solarized.css';
import '../css/idea.css';
import '../css/nord.css';

import 'codemirror/mode/python/python.js';

export class WidgetCodeModel extends DOMWidgetModel {
  defaults() {
    return {
      ...super.defaults(),
      _model_name: WidgetCodeModel.model_name,
      _model_module: WidgetCodeModel.model_module,
      _model_module_version: WidgetCodeModel.model_module_version,
      _view_name: WidgetCodeModel.view_name,
      _view_module: WidgetCodeModel.view_module,
      _view_module_version: WidgetCodeModel.view_module_version,
      value: 'Hello World',
    };
  }

  initialize(...args: any[]) {
    const [attributes, options] = args;
    DOMWidgetModel.prototype.initialize.apply(this, [attributes, options]);
    this.attributes['function_body_id'] = _.uniqueId('function_body');
  }

  static serializers: ISerializers = {
    ...DOMWidgetModel.serializers,
    // Add any extra serializers here
  };

  static model_name = 'WidgetCodeModel';
  static model_module = MODULE_NAME;
  static model_module_version = MODULE_VERSION;
  static view_name = 'WidgetCodeView'; // Set to null if no view
  static view_module = MODULE_NAME; // Set to null if no view
  static view_module_version = MODULE_VERSION;
}

export class WidgetCodeView extends DOMWidgetView {
  private mySignatureCodeMirror: any;
  private myDocstringCodeMirror: any;
  private myBodyCodeMirror: any;
  private theme: string = this.model.get('code_theme');

  render() {
    const cssStyles =
      '<style>\
            .forced-indent {width: 4em;}\
            .CodeMirror {border: 1px solid #aaa; height: auto;}\
            .CodeMirror.widget-code-input-signature {border-bottom: 0px;}\
            .CodeMirror.widget-code-input-docstring {border-top: 0px;}\
            .CodeMirror.widget-code-input-body {border-top: 0px;}\
        </style>';

    const theTextareaId = this.model.get('function_body_id');

    this.el.innerHTML =
      cssStyles +
      '<textarea id="' +
      theTextareaId +
      '-signature"></textarea>' +
      '<textarea id="' +
      theTextareaId +
      '-docstring"></textarea>' +
      '<textarea id="' +
      theTextareaId +
      '-body"></textarea>';

    _.defer(() => {
      this.mySignatureCodeMirror = CodeMirror.fromTextArea(
        <HTMLTextAreaElement>(
          document.getElementById(theTextareaId + '-signature')
        ),
        {
          lineNumbers: true,
          firstLineNumber: 1,
          mode: {
            name: 'python',
            version: 3,
            singleLineStringErrors: true,
            matchBrackets: true,
          },
          readOnly: true,
          indentUnit: 4,
          indentWithTabs: false,
        }
      );

      this.mySignatureCodeMirror.setOption('theme', this.theme);
      this.mySignatureCodeMirror.display.wrapper.classList.add(
        'widget-code-input-signature'
      );

      this.myDocstringCodeMirror = CodeMirror.fromTextArea(
        <HTMLTextAreaElement>(
          document.getElementById(theTextareaId + '-docstring')
        ),
        {
          lineNumbers: true,
          //firstLineNumber: // This will be set later by changing the content, this
          // indirectly calls the function to update the line numbers
          mode: {
            name: 'python',
            version: 3,
            singleLineStringErrors: true,
            matchBrackets: true,
          },
          readOnly: true,
          indentUnit: 4,
          indentWithTabs: false,
          gutters: ['CodeMirror-linenumbers', 'forced-indent'],
        }
      );
      // Add CSS
      this.myDocstringCodeMirror.setOption('theme', this.theme);
      this.myDocstringCodeMirror.display.wrapper.classList.add(
        'widget-code-input-docstring'
      );

      this.myBodyCodeMirror = CodeMirror.fromTextArea(
        <HTMLTextAreaElement>document.getElementById(theTextareaId + '-body'),
        {
          lineNumbers: true,
          //firstLineNumber: // This will be set later by changing the content, this
          // indirectly calls the function to update the line numbers
          mode: {
            name: 'python',
            version: 3,
            singleLineStringErrors: true,
            matchBrackets: true,
          },
          readOnly: false,
          indentUnit: 4,
          indentWithTabs: false,
          smartIndent: true,
          gutters: ['CodeMirror-linenumbers', 'forced-indent'],
        }
      );

      this.myBodyCodeMirror.setOption("extraKeys", {
        Tab: (cm: any) => {
          if (cm.getMode().name === 'null') {
            cm.execCommand('insertTab');
          } else {
            if (cm.somethingSelected()) {
              cm.execCommand('indentMore');
            } else {
              cm.execCommand('insertSoftTab');
            }
          }
        },
        Backspace: (cm: any) => {
          if (!cm.somethingSelected()) {
            let cursorsPos = cm.listSelections().map((selection: any) => selection.anchor);
            let indentUnit = cm.options.indentUnit;
            let shouldDelChar = false;
            for (let cursorIndex in cursorsPos) {
              let cursorPos = cursorsPos[cursorIndex];
              let indentation = cm.getStateAfter(cursorPos.line).indented;
              if (!(indentation !== 0 &&
                cursorPos.ch <= indentation &&
                cursorPos.ch % indentUnit === 0)) {
                shouldDelChar = true;
              }
            }
            if (!shouldDelChar) {
              cm.execCommand('indentLess');
            } else {
              cm.execCommand('delCharBefore');
            }
          } else {
            cm.execCommand('delCharBefore');
          }
        },
        'Shift-Tab': (cm: any) => cm.execCommand('indentLess')
      })

      // Add CSS
      this.myBodyCodeMirror.setOption('theme', this.theme);
      this.myBodyCodeMirror.display.wrapper.classList.add(
        'widget-code-input-body'
      );
      // I need to attach the backboneView, since I'm going to use
      // a CodeMirror event rather than a backboneView, but
      // bodyChange needs to access the Backbone.js View
      this.myBodyCodeMirror._backboneView = this;

      // Set the initial values of the cells
      this.signatureValueChanged();
      this.docstringValueChanged();
      this.bodyValueChanged();

      // When the value is changed in python, update the value in the widget
      // I set this in the 'defer' so that bodyValueChanged is called only
      // when the CodeWidget has been  rendered.
      this.model.on('change:function_name', this.signatureValueChanged, this);
      this.model.on(
        'change:function_parameters',
        this.signatureValueChanged,
        this
      );
      this.model.on('change:docstring', this.docstringValueChanged, this);
      this.model.on('change:function_body', this.bodyValueChanged, this);
      this.model.on('change:code_theme', this.themeChanged, this);

      // For a proper CodeMirror functioning, we use CodeMirror events
      // rather than Backbone.js events.
      // Only the function_body is not read-only, so we need only this
      // event
      this.myBodyCodeMirror.on('change', this.bodyChange);
    });
  }

  themeChanged() {
    this.theme = this.model.get('code_theme');
    this.mySignatureCodeMirror.setOption('theme', this.theme);
    this.myDocstringCodeMirror.setOption('theme', this.theme);
    this.myBodyCodeMirror.setOption('theme', this.theme);
  }

  bodyChange(instance: any, changeObj: any) {
    // On change of the text in the code widget, send back the content
    // to python.
    // Note that this event is triggered by backbone, so "this" is
    // NOT this class but the window; I need to access the view by
    // instance._backboneView that I attached when rendering.
    const currentFunctionBody = instance.getValue();
    instance._backboneView.model.set('function_body', currentFunctionBody);
    instance._backboneView.touch();
    console.log('Done');
  }

  signatureValueChanged() {
    // Set the value from python into the CodeMirror widget in the
    // frontend.
    const newSignature =
      'def ' +
      this.model.get('function_name') +
      '(' +
      this.model.get('function_parameters') +
      '):';
    if (newSignature !== this.mySignatureCodeMirror.getValue()) {
      this.mySignatureCodeMirror.setValue(newSignature);
    }
  }

  docstringValueChanged() {
    // Set the value from python into the CodeMirror widget in the
    // frontend.
    const newDocstring = '"""' + this.model.get('docstring') + '"""';
    if (newDocstring !== this.myDocstringCodeMirror.getValue()) {
      this.myDocstringCodeMirror.setValue(newDocstring);
    }
    this.updateLineNumbers();
  }

  bodyValueChanged() {
    // Set the value from python into the CodeMirror widget in the
    // frontend. Update it only if the content has changed:
    // typing re-fires this event, that would then change the content
    // of the widget, moving the cursor back to the top of the cell
    if (this.model.get('function_body') !== this.myBodyCodeMirror.getValue()) {
      this.myBodyCodeMirror.setValue(this.model.get('function_body'));
    }
  }

  updateLineNumbers() {
    const linesSignature = this.mySignatureCodeMirror
      .getValue()
      .split('\n').length;
    const linesDocstring = this.myDocstringCodeMirror
      .getValue()
      .split('\n').length;
    this.myDocstringCodeMirror.setOption('firstLineNumber', linesSignature + 1);
    this.myBodyCodeMirror.setOption(
      'firstLineNumber',
      linesDocstring + linesSignature + 1
    );
  }
}
